
----------------------------------------------------------------
----------------------------------------------------------------
---   Video Data Generation						--------
---														--------
----------------------------------------------------------------
----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity vga_sync is
PORT(   
		--Counter/VGA Timing
		clk							: IN STD_LOGIC;
		
		--Signal Syncs (Outputs)		
		hsync,
		vsync 						: OUT STD_LOGIC;
		
		clk_out_9mhz 				: OUT STD_LOGIC;		
		
		red, green, blue 			: OUT STD_LOGIC_VECTOR(5 DOWNTO 0);		
		
		led_1,
		led_2 						: OUT STD_LOGIC;
		
		pixel_row, pixel_column	: OUT STD_LOGIC_VECTOR(9 DOWNTO 0));
end vga_sync;

architecture vga_behavioral of vga_sync is

			--Sync Signals
SIGNAL 	h_sync, v_sync	:	STD_LOGIC;
			--Video Enables
SIGNAL 	video_en, 
		horizontal_en, 
		vertical_en	: STD_LOGIC;
		
			--Sync Counters
SIGNAL 	h_cnt, 
		v_cnt : STD_LOGIC_VECTOR(9 DOWNTO 0);

begin

	video_en <= horizontal_en AND vertical_en;
	
PROCESS
variable cnt: integer range 0 to 100000000;
variable speed: integer range 0 to 100000000;
variable clk_count: integer range 0 to 100;

BEGIN

	WAIT UNTIL(clk'EVENT) AND (clk = '1');
	
	
	--Blink Leds at 2Hz
	IF(speed >= 50000000) THEN
	led_2 <= '1';
	led_1 <= '0';		
	end if;
	IF(speed < 50000000) THEN
	led_2 <= '0';
	led_1 <= '1';
	end if;

			--Generate Horizontal Data
					--90 Columns Of Red
					--91 Columns Of Green
					--91 Rows Of Blue
		IF v_cnt <= 89 THEN		
			red 	<= "111111";
			green <= "000000";
			blue 	<= "000000";
		ELSIF v_cnt >= 90 and v_cnt <= 180 then
			red 	<= "000000";
			green <= "111111";
			blue 	<= "000000";
		ELSIF v_cnt >= 181 and v_cnt <= 271 then
			red 	<= "000000";
			green <= "000000";
			blue 	<= "111111";		
		ELSE
			red 	<= "111111";
			green <= "111111";
			blue 	<= "111111";			
		END IF;	
	
	--Clock Speed 100MHz
	--Slow Clock to 6 Cycles logic '0', 6 Cycles logic '1'
		-- (12/100,000,000) = 8.333MHz
						--LCD Rated Min Frequency = 7.83 MHz 
						--LCD Rated Typ Frequency = 9.00 MHz
						--LCD Rated Max Frequency = 9.26 MHz	
						
	IF(clk_count = 5) THEN
	clk_out_9mhz <= '0';
	end if;
	IF(clk_count = 11) THEN
	clk_out_9mhz <= '1';
	clk_count := 0;
	end if;
	
	clk_count:= 1 + clk_count;
	
	--Vga Timing Logic
	
	IF(clk_count = 1)THEN		
	--Horizontal Sync
	
		--Reset Horizontal Counter
	IF (h_cnt = 525) THEN
		h_cnt <= "0000000000";
	ELSE
		h_cnt <= h_cnt + 1;
	END IF;
	
		--Generate Horizontal Sync
	IF (h_cnt <= 523) AND (h_cnt >= 482) THEN
		h_sync <= '0';
	ELSE
		h_sync <= '1';
	END IF;
	
	--Vertical Sync

		--Reset Vertical Counter
	IF (v_cnt >= 285) AND (h_cnt >= 499) THEN
		v_cnt <= "0000000000";
	ELSIF (h_cnt = 499) THEN
		v_cnt <= v_cnt + 1;
	END IF;
	
		--Generate Vertical Sync
	IF (v_cnt <= 283) AND (v_cnt >= 274) THEN
		v_sync <= '0';	
	
	ELSE
		v_sync <= '1';
	END IF;
	
		--Generate Horizontal Data
	IF (h_cnt <= 479) THEN
		horizontal_en <= '1';
		pixel_column <= (h_cnt-20); --Does Nothing In This Design
	ELSE
		horizontal_en <= '0';
	END IF;
	
		--Generate Vertical Data
	IF (v_cnt <= 271) THEN
		vertical_en <= '1';
		pixel_row <= (v_cnt+20); --Does Nothing In This Design
	ELSE
		vertical_en <= '0';
	END IF;
		
	hsync	<= h_sync;
	vsync	<= v_sync;
		
	cnt := 0;

	ELSE
	cnt := cnt  + 1;
	speed := speed + 1;
END IF;		
	
END PROCESS;	

end vga_behavioral;